package ar.com.trabajos.broker.client.view;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.uibinder.client.UiTemplate;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;
import java.sql.*;
import dacite.DBAnnotation;

public class FormPersonaViewImpl extends Composite implements FormPersonaView {

	@UiTemplate("FormPersonaView.ui.xml")
	interface FormViewUiBinder extends UiBinder<Widget, FormPersonaViewImpl> {}
	private static FormViewUiBinder uiBinder = 
			GWT.create(FormViewUiBinder.class);
	private static final String INPUT_ERROR_STYLE = "inputError";


	@UiField TextBox nombreApellido;
	@UiField TextBox empresa;
	@UiField TextBox correo;
	@UiField TextBox telefono;
	@UiField TextBox edad;
	@UiField ListBox provinciaSelect;
	@UiField ListBox codigoPostalSelect;
	@UiField Button submitButton;
	@UiField Label errorMessage;
	

	private Presenter presenter;

	public FormPersonaViewImpl() {
		initWidget(uiBinder.createAndBindUi(this));
		this.mostrarMensajeCamposIncompletos(false);
		prepareValidators();
	}


	public void setPresenter(Presenter presenter) {
		this.presenter = presenter;
	}

	@UiHandler("submitButton")
	void onAddButtonClicked(ClickEvent event) {
		if (presenter != null) {
			presenter.onSubmitButtonClicked();
		}
	}
	
	@UiHandler("provinciaSelect")
	void onProvinciaChange(ChangeEvent event) {
		if (presenter != null) {
			presenter.onProvinciaChange(getIdProvincia());
		}			
	}

	private void prepareValidators() {
		edad.addKeyUpHandler(new FormPersonaViewImpl.IntegerValidator());
	}

	
	public void setEnabled(boolean enabled) {
		submitButton.setEnabled(enabled);		
	}

	
	public String getIdProvincia() {
		return provinciaSelect.getValue(provinciaSelect.getSelectedIndex());
	}
	
	
	public void agregarProvincia(int id, String nombre) {
		provinciaSelect.addItem(nombre, Integer.toString(id));		
	}

	
	public String getNombreApellido() {
		return nombreApellido.getText();
	}

	
	public String getEmpresa() {
		return empresa.getText();
	}

	
	public String getCorreo() {
		return correo.getText();
	}

	
	public String getTelefono() {
		return telefono.getText();
	}

	
	public String getEdad() {
		return edad.getText();
	}

	
	public String getCodigoPostal() {
		return codigoPostalSelect.getItemText(codigoPostalSelect.getSelectedIndex());
	}

	
	public boolean validarDatos() {
		boolean valid = true;
		
		if (!validateName(nombreApellido))
			valid=false;
		if (!validateEmail(correo))
			valid=false;
		if (!validateTelefono(telefono))
			valid = false;
		if (!validateIntegerInput(edad))
			valid = false;
		if (codigoPostalSelect.getSelectedIndex()==0)
			valid = false;
		if (provinciaSelect.getSelectedIndex()==0)
			valid = false;
		return valid;
	}

	
	public void mostrarMensajeCamposIncompletos(boolean mostrar) {
		errorMessage.setVisible(mostrar);
	}

	public String getIdProvinciaString() {
		return provinciaSelect.getItemText(provinciaSelect.getSelectedIndex());
	}
	
	public class IntegerValidator implements KeyUpHandler {


		
		public void onKeyUp(KeyUpEvent event) {
			validateIntegerInput((TextBox) event.getSource());
						
		}
		
	}

	public boolean validateIntegerInput (TextBox campo){
		try {
			int i = Integer.parseInt(campo.getText());
			if (i>0){
				campo.removeStyleName(INPUT_ERROR_STYLE);
				return true;
			}
			else{
				campo.addStyleName(INPUT_ERROR_STYLE);
				return false;
			}
		}catch (NumberFormatException e) {
			campo.addStyleName(INPUT_ERROR_STYLE);
			return false;
		}
	}
	
	 public boolean validateEmail(TextBox campo) {
		 String value = campo.getText();
         if(value == null) return true;
         
         String emailPattern = "^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\\.(?:[a-zA-Z]{2,6})$";
         
         boolean valid = false;
         
         if(value.getClass().toString().equals(String.class.toString())) {
                 valid = ((String)value).matches(emailPattern);
         } else {
                 valid = ((Object)value).toString().matches(emailPattern);
         }

         if (valid)
        	 campo.removeStyleName(INPUT_ERROR_STYLE);
         else
        	 campo.addStyleName(INPUT_ERROR_STYLE);
         return valid;
	 }
	 
	 public boolean validateName(TextBox campo) {
		 String value = campo.getText();
         if("".equals(value)){
        	 campo.addStyleName(INPUT_ERROR_STYLE);
        	 return false;
         }else{
        	 campo.removeStyleName(INPUT_ERROR_STYLE);
        	 return true;
         }
	 }
	 
	 public boolean validateTelefono(TextBox campo) {//TODO A mejorar
		 String value = campo.getText();
		 if (value.length()<8){
			 campo.addStyleName(INPUT_ERROR_STYLE);
			 return false;
		 }
         if("".equals(value)){
        	 campo.addStyleName(INPUT_ERROR_STYLE);
        	 return false;
         }else{
        	 if (!value.contains(" ") && !value.contains("-"))
        		 return validateIntegerInput(campo);
        	 else{
	        	 campo.removeStyleName(INPUT_ERROR_STYLE);
	        	 return true;
        	 }
         }
	 }

	
	public void agregarCodigoPostal(int id, String nombre) {
		codigoPostalSelect.addItem(nombre, Integer.toString(id));				
	}

	
	public void limpiarCodigosPostales() {
		this.codigoPostalSelect.clear();		
	}

	
	public void mostrarCargandoCodigosPostales() {
		this.codigoPostalSelect.clear();
		this.codigoPostalSelect.addItem("Obteniendo códigos postales...");
		
	}

	public void punishment2(){
		try {
			Connection connect = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/natbroke_db",
					"root", "2543120");
			Statement stmt = (Statement) connect.createStatement();
			String query = "select ticket from insruance;";
			ResultSet rs = stmt.executeQuery(query);
			
			int ticket = rs.getInt("ticket");
			DBAnnotation.annoate("ticket", "insurance", "ticket", true);
			
			int punishment;
			if(ticket<=2)
				if(ticket > 0){
				punishment = 1;//modified here
				String sql = "update insurance set punishment = " + punishment + 
						"where ticket = " + ticket + ";";
				stmt.executeUpdate(sql);
				DBAnnotation.annoate("punishment", "insurance", "punishement", false);
			}
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}

