package ar.com.trabajos.broker.server.service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.xml.ws.WebServiceException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Node;

import ar.com.libertyargentina.CotizarResponse.CotizarResult;
import ar.com.libertyargentina.Service;
import ar.com.trabajos.broker.server.dao.AppDAO;
import ar.com.trabajos.broker.server.utils.CodInfoAuto;
import ar.com.trabajos.broker.shared.entities.RequestDTO;
import ar.com.trabajos.broker.shared.entities.ResponseDTO;
import ar.com.trabajos.broker.shared.enums.BrokerEnum;
import ar.com.trabajos.broker.shared.enums.CoberturasEnum;

import com.sun.org.apache.xerces.internal.dom.ElementNSImpl;


public class ObtenerCotizacionLiberty implements Cotizador  {
    private static Log log = LogFactory.getLog(ObtenerCotizacionLiberty.class);
    private List<PedirCotizacionLibertyThread> threads= new ArrayList<ObtenerCotizacionLiberty.PedirCotizacionLibertyThread>();
	//Esto puede ser 6 o 12 dependiendo de si se usa vigencia mensual o semestral.
	private static int DIVISOR = 6;
	public ResponseDTO cotizar(RequestDTO input) {
		AppDAO dao = new AppDAO();
		ResponseDTO response = new ResponseDTO();
		response.setBroker(BrokerEnum.LIBERTY);
		HashMap<String, String> datosParaXML= new HashMap<String, String>();
		String codProvincia = String.format("%02d",dao.obtenerCodigoProvincia(input.getProvincia(), BrokerEnum.LIBERTY));
		datosParaXML.put("codProvincia", codProvincia);
		int codigoLocalidad = dao.obtenerCodigoLocalidadLiberty(input.getCodigoPostal());
		String codLocalidad = String.format("%06d",codigoLocalidad);
		datosParaXML.put("codLocalidad", codLocalidad);
		if (codigoLocalidad==-1){
			//TODO codigoLocalidad=999999;
			datosParaXML.put("codLocalidad", "1085");
			datosParaXML.put("codProvincia", "02");
		}
		Calendar cal = Calendar.getInstance();
		String fechaNacimiento = "01/01/" + Integer.toString( cal.get(Calendar.YEAR) - Integer.parseInt(input.getEdad()));
		datosParaXML.put("fechaNacimiento", fechaNacimiento);			
		int marcaCod = Integer.parseInt(CodInfoAuto.getMarcaFromCod(input.getCodInfoAuto()));
		int modeloAuto= Integer.parseInt(CodInfoAuto.getModeloFromCod(input.getCodInfoAuto()));
		datosParaXML.put("codMarca", Integer.toString(marcaCod));
		datosParaXML.put("codModelo", String.format("%03d",modeloAuto));
		int codUso = dao.obtenerCodigoUso(input.getTipoUso(), BrokerEnum.LIBERTY);
		datosParaXML.put("destino", Integer.toString(codUso));
		//Asume fecha de hoy
		SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
		datosParaXML.put("fechaVigencia", formateador.format(new Date()));
		//Se obtienen los productos disponibles

		HashMap<Integer, String> productos = dao.obtenerProductos(BrokerEnum.LIBERTY);
		PedirCotizacionLibertyThread thread1;
		for (Integer key : productos.keySet()) {
			threads.add(new PedirCotizacionLibertyThread(response, datosParaXML, key, input, false));
			
		}
		threads.add(new PedirCotizacionLibertyThread(response, datosParaXML, 54, input, true));
		
		for (PedirCotizacionLibertyThread thread : threads) {
			try {
				thread.join();
				if (thread.getCobertura()!=null){
					switch (thread.getCobertura()) {
					case RESPONSABILIDAD_CIVIL:
						response.setRc(thread.getPrecio());
						break;
					case TERCEROS_COMPLETOS_FULL:
						response.setTcf(thread.getPrecio());					
						break;
					case TERCEROS_COMPLETOS_FULL_GRANIZO:
						response.setTcfg(thread.getPrecio());
						break;
					case TERCEROS_COPMLETOS:
						response.setTc(thread.getPrecio());
						break;
					case TODO_RIESGO:
						response.setTr(thread.getPrecio());
						break;
					case TOTAL_SIN_DANOS_PARCIALES:
						response.setTt(thread.getPrecio());
						break;
					}
				}
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		//TODO Arreglar esto:
		/*
		if (response.getRc()==3)
			response.setRc(0);
		if (response.getTc()==3)
			response.setTc(0);
		if (response.getTcf()==3)
			response.setTcf(0);
		if (response.getTcfg()==3)
			response.setTcfg(0);
		if (response.getTr()==3)
			response.setTr(0);
		if (response.getTt()==3)
			response.setTt(0);
			*/
		//FIN arreglar
		
		if (response.getTc()!=0 && response.getTcf()!=0 &&  response.getTcfg()!=0 ){
			int[] intArray = new int[] {response.getTc(), response.getTcf(), response.getTcfg()};
			Arrays.sort(intArray);
			if (intArray[0] == intArray[1])
				intArray[1]=intArray[2];
			response.setTc(intArray[0]);
			response.setTcf(intArray[1]);
			response.setTcfg(intArray[2]);
		}
		
		return response;
	}
	
	
	private class PedirCotizacionLibertyThread extends Thread {
		
		private boolean granizo;
		private int key;
		CotizarResult libertyResponse=null;
		private String datosInput;
		private int precio= -1;
		private CoberturasEnum cobertura= null;
		
		public PedirCotizacionLibertyThread(ResponseDTO response, HashMap<String, String> datosParaXML, int key, RequestDTO input, boolean granizo){
			this.granizo=granizo;
			datosParaXML.put("codProducto", String.format("%04d", key));
			datosInput = XMLBuilder.buildXMLLiberty(input, datosParaXML, granizo);
			this.key=key;
			this.start();
		}
		
		public void run() {
			try {
				Service libertyService = new Service();
				libertyResponse = libertyService.getServiceSoap().cotizar(datosInput);
				ElementNSImpl element = (ElementNSImpl) libertyResponse.getContent().get(0);
				org.w3c.dom.NodeList nodeList = element.getChildNodes();

				for (int i = 0; i < nodeList.getLength(); i++) {
					Node node = (Node) nodeList.item(i);
					if (node.getNodeName().equals("Errores")){
						//Ignorar error: Error al ejecutar el proceso de calculo: clsInterfaceProceso.Run: clsProceso.Run: (13) Type mismatch!
						if (!node.getTextContent().equals("Error al ejecutar el proceso de calculo: clsInterfaceProceso.Run: clsProceso.Run: (13) Type mismatch!")){
							log.error("Liberty error:" + node.getTextContent());
						}
					}else if (node.getNodeName().equals("Premio")){
						int resultado = -1;
						try {
							if (node.getTextContent().contains(".")){
								resultado = (Integer.parseInt(node.getTextContent().substring(0, node.getTextContent().indexOf(".")))/DIVISOR);
							}else{
								resultado = (Integer.parseInt(node.getTextContent())/DIVISOR);
							}
							if (resultado != 3){
								setPrecio(resultado);
								if (key == 64){
									setCobertura(CoberturasEnum.RESPONSABILIDAD_CIVIL);
								}else if (key == 50){
									setCobertura(CoberturasEnum.TOTAL_SIN_DANOS_PARCIALES);
								}else if (key == 53){
									setCobertura(CoberturasEnum.TERCEROS_COPMLETOS);
								}else if (key == 54){
									if (granizo)
										setCobertura(CoberturasEnum.TERCEROS_COMPLETOS_FULL_GRANIZO);
									else
										setCobertura(CoberturasEnum.TERCEROS_COMPLETOS_FULL);
								}else if (key == 689){
									setCobertura(CoberturasEnum.TODO_RIESGO);
								}else if (key == 688){
									setCobertura(CoberturasEnum.TODO_RIESGO);
								}
							}
							
						} catch (NumberFormatException e) {
							log.error("Error parseando precio Liberty: " + e.getMessage()+ ". Precio: " + resultado );
						}
					}

					//System.out.println(node.getNodeName());
					//System.out.println(node.getTextContent());
				}
			} catch (WebServiceException e) {
				log.error("El servicio de liberty esta caido. El wsdl no se encuentra.");
					System.err.println("El servicio de liberty esta caido. El wsdl no se encuentra.");
			}
		}
		public int getPrecio() {
			return precio;
		}
		public void setPrecio(int precio) {
			this.precio = precio;
		}
		public CoberturasEnum getCobertura() {
			return cobertura;
		}
		public void setCobertura(CoberturasEnum cobertura) {
			this.cobertura = cobertura;
		}
		
	}

}
